import { ZERO } from '@/constants/common';
import { bioEntityContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { initialStateFixture } from '@/redux/drawer/drawerFixture';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { renderHook } from '@testing-library/react';
import { useHandlePinIconClick } from './useHandlePinIconClick';

describe('useHandlePinIconClick - util', () => {
  describe('when pin clicked', () => {
    describe('when tab is invalid', () => {
      const pinId = 123;
      const { Wrapper, store } = getReduxStoreWithActionsListener({
        ...INITIAL_STORE_STATE_MOCK,
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                    id: pinId,
                    model: ZERO,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
      });

      const {
        result: { current: onPinIconClick },
      } = renderHook(() => useHandlePinIconClick(), {
        wrapper: Wrapper,
      });

      onPinIconClick({ id: pinId });

      it('should do not dispatch any action', () => {
        const actions = store.getActions();

        expect(actions).toStrictEqual([]);
      });
    });

    describe('when tab is already selected', () => {
      const pinId = 123;
      const { Wrapper, store } = getReduxStoreWithActionsListener({
        ...INITIAL_STORE_STATE_MOCK,
        drawer: {
          ...initialStateFixture,
          drawerName: 'search',
          isOpen: true,
          searchDrawerState: {
            ...initialStateFixture.searchDrawerState,
            selectedSearchElement: 'search-tab',
          },
        },
        bioEntity: {
          data: [
            {
              searchQueryElement: 'search-tab',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                    id: pinId,
                    model: ZERO,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
      });

      const {
        result: { current: onPinIconClick },
      } = renderHook(() => useHandlePinIconClick(), {
        wrapper: Wrapper,
      });

      onPinIconClick({ id: pinId });

      it('should do not dispatch any action', () => {
        const actions = store.getActions();

        expect(actions).toStrictEqual([]);
      });
    });

    describe('when pin is marker', () => {
      const pinId = 'af57c6ce-8b83-47e4-a7eb-9511634c7c5f';
      const { Wrapper, store } = getReduxStoreWithActionsListener({
        ...INITIAL_STORE_STATE_MOCK,
        drawer: {
          ...initialStateFixture,
          drawerName: 'search',
          isOpen: true,
          searchDrawerState: {
            ...initialStateFixture.searchDrawerState,
            selectedSearchElement: 'search-tab',
          },
        },
        bioEntity: {
          data: [
            {
              searchQueryElement: 'search-tab',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                    id: pinId,
                    model: ZERO,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
      });

      const {
        result: { current: onPinIconClick },
      } = renderHook(() => useHandlePinIconClick(), {
        wrapper: Wrapper,
      });

      onPinIconClick({ id: pinId });

      it('should do not dispatch any action', () => {
        const actions = store.getActions();

        expect(actions).toStrictEqual([]);
      });
    });

    describe('when all valid', () => {
      const pinId = 123;
      const { Wrapper, store } = getReduxStoreWithActionsListener({
        ...INITIAL_STORE_STATE_MOCK,
        drawer: {
          ...initialStateFixture,
          drawerName: 'search',
          isOpen: true,
          searchDrawerState: {
            ...initialStateFixture.searchDrawerState,
            selectedSearchElement: 'search-tab-2',
          },
        },
        bioEntity: {
          data: [
            {
              searchQueryElement: 'search-tab',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                    id: pinId,
                    model: ZERO,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
      });

      const {
        result: { current: onPinIconClick },
      } = renderHook(() => useHandlePinIconClick(), {
        wrapper: Wrapper,
      });

      onPinIconClick({ id: pinId });

      it('should dispatch action', () => {
        const actions = store.getActions();
        expect(actions).toStrictEqual([{ payload: 'search-tab', type: 'drawer/selectTab' }]);
      });
    });
  });
});
